"""
Place the well images of tiled fields in a layout similar to a multiwell
plate. This makes it easy to get an overview of the different conditions in
the plate.

positional arguments:
  path                  path to well images (default: current directory)

optional arguments:
  -h, --help            show this help message and exit
  -i INPUT_FORMAT, --input-format INPUT_FORMAT
                        format for images to be tiled (default: jpg)
  -o OUTPUT_FORMAT, --output-format OUTPUT_FORMAT
                        format for the tiled output image (default: jpg)
"""
import argparse
from pathlib import Path
from datetime import datetime

from joblib import Parallel, delayed

from .tile_images import tile_images

def main():
    # `__doc__.split('\n\n')[0]` everything until the first newline in the docstring
    parser = argparse.ArgumentParser(description=__doc__.split('\n\n')[0],
        formatter_class=argparse.RawTextHelpFormatter)
    parser.add_argument('path', default='./', nargs='?',
        help='path to well images (default: current directory)')
    parser.add_argument('-i', '--input-format', default='jpg',
        help='format for images to be tiled (default: %(default)s)')
    parser.add_argument('-o', '--output-format', default='jpg',
        help='format for the tiled output image (default: %(default)s)')

    args = parser.parse_args()
    output_format = args.output_format.lower()
    input_format = args.input_format.lower()
    img_dir = Path(args.path)
    channels = {x.name.split('-')[0] for x in img_dir.iterdir()
        if x.name.endswith(input_format)}
    path_match_string = '{}' + '*.{}'.format(input_format)
    well_dir = img_dir.joinpath(path_match_string)
    tiled_img_dir = img_dir.resolve().parent.joinpath('tiled-well-images')
    tiled_img_dir.mkdir(exist_ok=True)
    timestamp = datetime.now().strftime('%Y%m%d-%H%M%S')
    timestamp_dir = tiled_img_dir.joinpath(timestamp)
    timestamp_dir.mkdir()
    # Process each channel in parallel
    results = Parallel(n_jobs=-2)(delayed(tile_images) # Function
        (img_dir, channel, input_format, output_format, timestamp_dir) # Arguments
        for channel in channels) # Loop to parallelize


if __name__ == '__main__':
    main()



import matplotlib.pyplot as plt

def tile_images(img_dir, channel, input_format, output_format, timestamp_dir):
    '''Layout the well images in a grid representing the multiwell plate'''
    print(channel)
    # Prevent figure windows from showing up
    plt.ioff()
    img_names = sorted(list(img_dir.glob('{}-*.{}'.format(channel, input_format))))
    rows = set([img.name[2] for img in img_names])
    cols = set([int(img.name[3:5]) for img in img_names])
    fig, axes = plt.subplots(len(rows), len(cols), figsize=(len(cols)*4, len(rows)*4))
    for ax, img_name in zip(axes.ravel(), img_names):
        print(img_name.name)
        ax.axis('off')
        ax.imshow(plt.imread(img_name), cmap='gray', vmin=0, vmax=256)
#         ax.set_title(img_name.name, fontsize=20, color='white', y=0.5)
#     fig.tight_layout(pad=20, w_pad=50, h_pad=50, rect=[0, 0, 0.992, 0.993])
    fig.tight_layout(pad=0, w_pad=-3, h_pad=-2, rect=[0, 0, 0.992, 0.993])
    # Pyplot can't handle Path object yet, lands in 2.1
    # https://github.com/matplotlib/matplotlib/pull/8481
    print('Saving image...')
    fig.savefig(timestamp_dir.joinpath('ch{}.{}'.format(
        channel, output_format)).as_posix(), dpi=200)#, bbox_inches='tight', pad_inches=0)
    return None





from setuptools import setup

setup(
    # Needed to silence warnings (and to be a worthwhile package)
    name='tile_wells',
    url='https://github.com/joelostblom/tile_wells/',
    author='Joel Ostblom',
    author_email='joel.ostblom@gmail.com',
    # Needed to actually package something
    packages=['tile_wells'],
    scripts=['bin/tile_wells'],
    # Needed for dependencies
    install_requires=['matplotlib', 'joblib'],
    # *strongly* suggested for sharing
    version='0.11',
    # The license can be anything you like
    license='MIT',
    description='''Place the well images of tiled fields in a layout similar to a multiwell
plate. This makes it easy to get an overview of the different conditions in
the plate.''',
    # We will also need a readme eventually (there will be a warning)
    # long_description=open('README.txt').read(),
)
